{% extends "admin/base_site.html" %}
{% load i18n %}
{% block coltype %}colSM{% endblock %}
{% block breadcrumbs %}<div class="breadcrumbs"><a href="../../">Home</a> &rsaquo; <a href="../">Documentation</a> &rsaquo; filters</div>{% endblock %}
{% block title %}Template filters{% endblock %}

{% block content %}

<h1>Template filter documentation</h1>

<div id="content-main">
{% regroup filters|dictsort:"library" by library as filter_libraries %}
{% for library in filter_libraries %}
<div class="module">
    <h2>{% firstof library.grouper "Built-in filters" %}</h2>
    {% if library.grouper %}<p class="small quiet">To use these filters, put <code>{% templatetag openblock %} load {{ library.grouper }} {% templatetag closeblock %}</code> in your template before using the filter.</p><hr />{% endif %}
    {% for filter in library.list|dictsort:"name" %}
    <h3 id="{{ filter.name }}">{{ filter.name }}</h3>
    <p>{{ filter.title }}</p>
    <p>{{ filter.body }}</p>
    {% if not forloop.last %}<hr />{% endif %}
    {% endfor %}
</div>
{% endfor %}
</div>

{% endblock %}

{% block sidebar %}

<div id="content-related">

{% regroup filters|dictsort:"library" by library as filter_libraries %}
{% for library in filter_libraries %}
<div class="module">
    <h2>{% firstof library.grouper "Built-in filters" %}</h2>
    <ul>
    {% for filter in library.list|dictsort:"name" %}
        <li><a href="#{{ filter.name }}">{{ filter.name }}</a></li>
    {% endfor %}
    </ul>
</div>
{% endfor %}

</div>

{% endblock %}
