{% extends "comments/base.html" %}
{% load i18n %}

{% block title %}{% trans "Thanks for commenting" %}.{% endblock %}

{% block content %}
  <h1>{% trans "Thank you for your comment" %}.</h1>
{% endblock %}
