{% extends "comments/base.html" %}
{% load i18n %}

{% block title %}{% trans "Thanks for removing" %}.{% endblock %}

{% block content %}
  <h1>{% trans "Thanks for taking the time to improve the quality of discussion on our site" %}.</h1>
{% endblock %}
