# -*- coding: utf-8 -*-
"""
A list of Argentinean provinces and autonomous cities as `choices` in a
formfield. From
http://www.argentina.gov.ar/argentina/portal/paginas.dhtml?pagina=425

This exists in this standalone file so that it's only imported into memory
when explicitly needed.
"""

PROVINCE_CHOICES = (
    ('B', u'Buenos Aires'),
    ('K', u'Catamarca'),
    ('H', u'Chaco'),
    ('U', u'Chubut'),
    ('C', u'Ciudad Autónoma de Buenos Aires'),
    ('X', u'Córdoba'),
    ('W', u'Corrientes'),
    ('E', u'Entre Ríos'),
    ('P', u'Formosa'),
    ('Y', u'Jujuy'),
    ('L', u'La Pampa'),
    ('F', u'La Rioja'),
    ('M', u'Mendoza'),
    ('N', u'Misiones'),
    ('Q', u'Neuquén'),
    ('R', u'Río Negro'),
    ('A', u'Salta'),
    ('J', u'San Juan'),
    ('D', u'San Luis'),
    ('Z', u'Santa Cruz'),
    ('S', u'Santa Fe'),
    ('G', u'Santiago del Estero'),
    ('V', u'Tierra del Fuego, Antártida e Islas del Atlántico Sur'),
    ('T', u'Tucumán'),
)
