{% extends "admin/base_site.html" %}
{% load i18n %}

{% block breadcrumbs %}<div class="breadcrumbs"><a href="../">{% trans 'Home' %}</a></div>{% endblock %}

{% block content %}

<p>{% trans "Thanks for spending some quality time with the Web site today." %}</p>

<p><a href="../">{% trans 'Log in again' %}</a></p>

{% endblock %}
